MODULE compute_temperature_mod
  USE icosa
  IMPLICIT NONE
  PRIVATE
  SAVE

  PUBLIC :: temperature, compute_temperature

CONTAINS
  
  SUBROUTINE temperature(f_pmid,f_q,f_temp)
    TYPE(t_field), POINTER :: f_pmid(:)         ! IN
    TYPE(t_field), POINTER :: f_q(:)            ! IN
    TYPE(t_field), POINTER :: f_temp(:)         ! INOUT
    
    REAL(rstd), POINTER :: pmid(:,:)
    REAL(rstd), POINTER :: q(:,:,:)
    REAL(rstd), POINTER :: temp(:,:)
    INTEGER :: ind
    
    DO ind=1,ndomain
       IF (.NOT. assigned_domain(ind)) CYCLE
       CALL swap_dimensions(ind)
       CALL swap_geometry(ind)
       pmid=f_pmid(ind)
       q=f_q(ind)
       temp=f_temp(ind)
       CALL compute_temperature(pmid,q,temp)
    END DO
  END SUBROUTINE temperature
  
  SUBROUTINE compute_temperature(pmid,q,temp)
    USE omp_para
    REAL(rstd),INTENT(IN)    :: pmid(iim*jjm,llm)
    REAL(rstd),INTENT(IN)    :: q(iim*jjm,llm,nqtot)
    REAL(rstd),INTENT(INOUT) :: temp(iim*jjm,llm)

    REAL(rstd) :: Rd, p_ik, theta_ik, temp_ik, qv, chi, Rmix
    INTEGER :: ij,l

    Rd = kappa*cpp
    DO l=ll_begin,ll_end
       DO ij=ij_begin,ij_end
          p_ik = pmid(ij,l)
          theta_ik = temp(ij,l)
          qv = q(ij,l,1) ! water vapor mixing ratio = mv/md
          SELECT CASE(caldyn_thermo)
          CASE(thermo_theta)
             temp_ik = theta_ik*((p_ik/preff)**kappa)
          CASE(thermo_entropy)
             temp_ik = Treff*exp((theta_ik + Rd*log(p_ik/preff))/cpp)
          CASE(thermo_moist)
             Rmix = Rd+qv*Rv
             chi = ( theta_ik + Rmix*log(p_ik/preff) ) / (cpp + qv*cppv) ! log(T/Treff)
             temp_ik = Treff*exp(chi)
          END SELECT
          IF(physics_thermo==thermo_fake_moist) temp_ik=temp_ik/(1+0.608*qv)
          temp(ij,l)=temp_ik
       END DO
    END DO
  END SUBROUTINE compute_temperature

  SUBROUTINE Tv2T(f_Tv, f_q, f_T)
    TYPE(t_field), POINTER :: f_TV(:)
    TYPE(t_field), POINTER :: f_q(:)
    TYPE(t_field), POINTER :: f_T(:)
    
    REAL(rstd),POINTER :: Tv(:,:), q(:,:,:), T(:,:)
    INTEGER :: ind
    
    DO ind=1,ndomain
       IF (.NOT. assigned_domain(ind)) CYCLE
       CALL swap_dimensions(ind)
       CALL swap_geometry(ind)
       Tv=f_Tv(ind)
       T=f_T(ind)
       SELECT CASE(physics_thermo)
       CASE(thermo_dry)
          T=Tv
       CASE(thermo_fake_moist)
          q=f_q(ind)
          T=Tv/(1+0.608*q(:,:,1))
       END SELECT
    END DO
  END SUBROUTINE Tv2T

END MODULE compute_temperature_mod
